#ifndef PARTICLEDATA_H
#define PARTICLEDATA_H

/*
    Copyright (C) 2010  Thomas Schrøder
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    LICENSE.txt file for license details.
*/


class ParticleData{
  
 private:
  ParticleData(const ParticleData&);
  
  unsigned int numParticles; // The number of real particles.
  unsigned int numVirtualParticles; // The number of 'particles' assigned threads (>=numParticles).
  unsigned int allocatedNumParticles; // Internal variable for managing dynamic allocation.  
  unsigned int numBlocks; // The number of blocks.

  void AllocateParticles(unsigned int nvp);
  void FreeParticles();
  bool verbose;
  
 public:
  ParticleData();
  ~ParticleData();
  ParticleData& operator=(const ParticleData& P);
  void CopyHostData(const ParticleData& P);
  void SetVerbose( bool vb) { verbose = vb; }

  // Host.
  float4* h_r;             // [x, y, z, type]
  float4* h_v;             // [vx, vy, vz, 1/mass]
  float4* h_f;             // [fx, fy, fz, u]
  float4* h_w;             // [w_vdw_mol, sxz, sxy, w_vdw_atom]
  float4* h_im;            // [imx, imy, imz, 0]
  float4* h_sts;           // [sxx, syy, szz, syz]
  float4* h_misc;          // [ConfigT]
  unsigned int* h_Type;    // Type (on cpu) 
  
  // Device.
  float4* d_r; 
  float4* d_v; 
  float4* d_f;
  float4* d_w;
  float4* d_im;
  float4* d_sts;
  float4* d_misc; 
 
  // Functions for transferring data to the device
  void CopyPosToDevice() const;
  void CopyVelToDevice() const;
  void CopyForToDevice() const;
  void CopyImagesToDevice() const;
  void CopyConfToDevice() const; // all of them together

  // Functions for transferring data from the device
  void CopyPosFromDevice() const;
  void CopyVelFromDevice() const;
  void CopyForFromDevice() const;
  void CopyVirFromDevice() const;
  void CopyImagesFromDevice() const;
  void CopyStressFromDevice() const;
  void CopyMiscFromDevice() const;
  void CopyConfFromDevice() const; // all of them together

  // Get methods
  unsigned int GetNumberOfParticles() const { return numParticles; }
  unsigned int GetNumberOfVirtualParticles() const { return numVirtualParticles; }
  unsigned int GetNumberOfBlocks() const { return numBlocks; }

  // Set methods
  void SetNumberOfParticles(unsigned int num_part, unsigned int pb, unsigned int tp);
};

#endif // PARTICLEDATA_H
